import comtypes
from os import path
from comtypes.client import CreateObject

def create_shortcut(outputfilename, targetfilename, description,
                    arguments="", workdir=None, iconfilename=None, iconindex=0):

    if not path.isfile(targetfilename):
        raise RuntimeError('Shortcut target "%s" does not exist' % targetfilename)
    if iconfilename and not path.isfile(iconfilename):
        raise RuntimeError('Shortcut icon file "%s" does not exist' % iconfilename)
    if workdir and not path.isdir(workdir):
        raise RuntimeError('Shortcut working directory "%s" does not exist' %
                           workdir)

    wsh_shell = CreateObject("WScript.Shell")
    from comtypes.gen import IWshRuntimeLibrary
    sc = comtypes.cast(wsh_shell.CreateShortcut(outputfilename),
                        comtypes.POINTER(IWshRuntimeLibrary.IWshShortcut))
    sc.TargetPath = path.abspath(targetfilename)
    sc.Description = description
    sc.Arguments = arguments
    if iconfilename:
        sc.IconLocation = path.abspath(iconfilename)
    if workdir:
        sc.WorkingDirectory = path.abspath(workdir)
    sc.Save()
