#    Copyright (C) 2012, Continuum Analytics
#    All rights reserved.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions are
#    met:
#
#        * Redistributions of source code must retain the above copyright
#           notice, this list of conditions and the following disclaimer.
#
#        * Redistributions in binary form must reproduce the above
#           copyright notice, this list of conditions and the following
#           disclaimer in the documentation and/or other materials provided
#           with the distribution.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys, os, shutil, json, re, traceback, ctypes
from os.path import isdir, join

import shortcut, csidl

var_dict = {}
var_dict['PYTHON_SCRIPTS'] = join(sys.prefix, 'Scripts')
var_dict['PYTHON_SITE_PACKAGES'] = join(sys.prefix, 'Lib', 'site-packages')
var_dict['USERPROFILE'] = csidl.get_folder_path('CSIDL_PROFILE')
var_dict['PERSONALDIR'] = csidl.get_folder_path('CSIDL_PERSONAL')
var_pat = re.compile(r'\${(\w+)}')

def error_dialog(msg, title):
    MB_ICONERROR = 0x00000010L
    MB_OK = 0
    ctypes.windll.user32.MessageBoxW(0, unicode(msg), unicode(title),
                                     MB_OK|MB_ICONERROR)

def substitute_variables(s):
    def var_repl(match):
        return var_dict[match.group(1)]
    return var_pat.sub(var_repl, s)

def make_menu_item_shortcut(menuitem, shortcut_dir):
    name = menuitem['name']
    workdir = substitute_variables(menuitem['workdir'])
    if not isdir(workdir):
        os.mkdir(workdir)
    icon = substitute_variables(menuitem['icon'])
    if menuitem.has_key('pyscript'):
        exe = join(sys.prefix, 'python.exe')
        args = substitute_variables(menuitem['pyscript'])
    elif menuitem.has_key('pywscript'):
        exe = join(sys.prefix, 'pythonw.exe')
        args = substitute_variables(menuitem['pywscript'])
    else:
        raise RuntimeError('Unrecognized Program Files shortcut command '
                           'menu item in %s' % menuitem)

    shortcut.create_shortcut(join(shortcut_dir, '%s.lnk' % name),
                             exe, '', args, workdir, icon)

def make_all_menus(allusers):
    path = join(sys.prefix, 'Lib', 'version.txt')
    try:
        program_folder = open(path).read().strip()
    except IOError:
        program_folder = 'Anaconda'

    if allusers:
        shortcut_dir = join(csidl.get_folder_path('CSIDL_COMMON_PROGRAMS'),
                            program_folder)
    else:
        shortcut_dir = join(csidl.get_folder_path('CSIDL_PROGRAMS'),
                            program_folder)

    # Create all the menu items from scratch
    if isdir(shortcut_dir):
        shutil.rmtree(shortcut_dir)
    os.mkdir(shortcut_dir)

    menus_dir = join(sys.prefix, 'Menu')
    for f in os.listdir(menus_dir):
        with open(join(menus_dir, f)) as fi:
            menu = json.load(fi)
        for menuitem in menu:
            make_menu_item_shortcut(menuitem, shortcut_dir)

if __name__ == '__main__':
    allusers = not (sys.argv.count('--just-me') > 0)
    if sys.argv.count('--no-uac') > 0 or not allusers:
        try:
            make_all_menus(allusers)
        except:
            error_dialog(traceback.format_exc(),
                         'Error Creating Anaconda Menu Items')
            sys.exit(1)
    else:
        from uac import uac_elevate_privileges
        sys.exit(uac_elevate_privileges(__file__,
                                        '--no-uac ' + ' '.join(sys.argv[1:])))
