#    Copyright (C) 2012, Continuum Analytics
#    All rights reserved.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions are
#    met:
#
#        * Redistributions of source code must retain the above copyright
#           notice, this list of conditions and the following disclaimer.
#
#        * Redistributions in binary form must reproduce the above
#           copyright notice, this list of conditions and the following
#           disclaimer in the documentation and/or other materials provided
#           with the distribution.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


__all__ = ['uac_elevate_privileges']

import sys, ctypes
from ctypes import wintypes
from os.path import abspath, join


class SHELLEXECUTEINFO(ctypes.Structure):
    _fields_ = [('cbSize', wintypes.DWORD),
                ('fMask', wintypes.ULONG),
                ('hwnd', wintypes.HWND),
                ('lpVerb', wintypes.LPCWSTR),
                ('lpFile', wintypes.LPCWSTR),
                ('lpParameters', wintypes.LPCWSTR),
                ('lpDirectory', wintypes.LPCWSTR),
                ('nShow', ctypes.c_int),
                ('hInstApp', wintypes.HINSTANCE),
                ('lpIDList', wintypes.LPVOID),
                ('lpClass', wintypes.LPCWSTR),
                ('hkeyClass', wintypes.HKEY),
                ('dwHotKey', wintypes.DWORD),
                ('hIcon', wintypes.HANDLE),
                ('hProcess', wintypes.HANDLE)]
SEE_MASK_NOCLOSEPROCESS = 0x00000040L
SEE_MASK_NOASYNC = 0x00000100L
SW_SHOWNORMAL = 1
INFINITE = 0xFFFFFFFF

ShellExecuteEx = ctypes.windll.shell32.ShellExecuteExW
ShellExecuteEx.restype = wintypes.BOOL
ShellExecuteEx.argtypes = [ctypes.POINTER(SHELLEXECUTEINFO)]
CloseHandle = ctypes.windll.kernel32.CloseHandle
CloseHandle.restype = wintypes.BOOL
CloseHandle.argtypes = [wintypes.HANDLE]
WaitForSingleObject = ctypes.windll.kernel32.WaitForSingleObject
WaitForSingleObject.restype = wintypes.DWORD
WaitForSingleObject.argtypes = [wintypes.HANDLE, wintypes.DWORD]
GetExitCodeProcess = ctypes.windll.kernel32.GetExitCodeProcess
GetExitCodeProcess.restype = wintypes.BOOL
GetExitCodeProcess.argtypes = [wintypes.HANDLE, ctypes.POINTER(wintypes.DWORD)]
SetEnvironmentVariable = ctypes.windll.kernel32.SetEnvironmentVariableW
SetEnvironmentVariable.restype = wintypes.BOOL
SetEnvironmentVariable.argtypes = [wintypes.LPCWSTR, wintypes.LPCWSTR]

def uac_elevate_privileges(scriptfile, arguments,
                           interpreter=abspath(join(sys.prefix, 'pythonw.exe'))):
    """Executes the requested python script with elevated UAC privileges.

        scriptfile - The full path to a .py script file to execute.
        arguments - The command line arguments, as a single string.
        interpreter - Defaults to the current python interpreter,
                      but override it to choose a different one.
    """
    shexifo = SHELLEXECUTEINFO()
    shexifo.cbSize = ctypes.sizeof(shexifo)
    shexifo.fMask = SEE_MASK_NOCLOSEPROCESS|SEE_MASK_NOASYNC
    shexifo.lpVerb = 'runas' # This triggers UAC elevation
    shexifo.lpFile = interpreter
    shexifo.lpParameters = u'"%s" %s' % (scriptfile, arguments)
    shexifo.nShow = SW_SHOWNORMAL

    # Force the PYTHONPATH to be empty, since in the Anaconda PyInstaller,
    # without this we get conflicting modules.
    if not SetEnvironmentVariable('PYTHONPATH', ''):
        raise ctypes.WinError()

    if not ShellExecuteEx(ctypes.byref(shexifo)):
        raise ctypes.WinError()
    if shexifo.hProcess is None:
        raise RuntimeError('ShellExecuteEx failed to provide a handle to '
                           'the process')
    try:
        # Wait for the process to finish
        if WaitForSingleObject(shexifo.hProcess, INFINITE):
            raise ctypes.WinError()
        # Pass along its return code
        retcode = wintypes.DWORD(1)
        if not GetExitCodeProcess(shexifo.hProcess, ctypes.byref(retcode)):
            raise ctypes.WinError()
        return retcode.value
    finally:
        CloseHandle(shexifo.hProcess)
